#include "nateon.h"
#include "buddy.h"
#include <cipher.h>

void nateon_buddy_destroy(NateBuddyData* Buddy)
{
	if(!Buddy) return;

	if(Buddy->nid)   g_free(Buddy->nid);
	if(Buddy->nick)  g_free(Buddy->nick);
	if(Buddy->name)  g_free(Buddy->name);
	if(Buddy->email) g_free(Buddy->email);
	if(Buddy->phone) g_free(Buddy->phone);
	if(Buddy->birth) g_free(Buddy->birth);
	if(Buddy->uid)   g_free(Buddy->uid);
	
	g_free(Buddy);
}

void nateon_group_destroy(NateGroupData* Group)
{
	if(!Group) return;

	if(Group->name) g_free(Group->name);
	
	g_free(Group);
}

void nateon_buddies_destroy(NateBuddy* NBuddy)
{
	GList* list;

	for(list = NBuddy->buddies; list != NULL; list = list->next)
	{
		nateon_buddy_destroy(list->data);
	}

	g_list_free(NBuddy->buddies);

	for(list = NBuddy->groups; list != NULL; list = list->next)
	{
		nateon_group_destroy(list->data);
	}

	g_list_free(NBuddy->groups);
	
	g_free(NBuddy);
}

void nateon_buddy_append_buddy(NateBuddy* NBuddy, NateBuddyData* Buddy)
{
	GaimGroup* gg = gaim_find_group(nateon_buddy_find_group_with_id(NBuddy, Buddy->gid)->name);
	GaimBuddy* gb = gaim_find_buddy_in_group(NBuddy->account, Buddy->nid, gg);
	
	if(!gb)
	{
		gb = gaim_buddy_new(NBuddy->account, Buddy->nid, NULL);

		gaim_blist_add_buddy(gb, NULL, gg, NULL);
	}
	
	if(!Buddy->isadd)
	{
		Buddy->isadd = TRUE;
	}

	gb->proto_data = Buddy;

	gaim_blist_node_set_string((GaimBlistNode*)gb, "protocol", "nateon");
	gaim_blist_node_set_string((GaimBlistNode*)gb, "uid", Buddy->uid);
}

void nateon_buddy_remove_buddy(NateBuddy* NBuddy, NateBuddyData* Buddy)
{
	NBuddy->buddies = g_list_remove(NBuddy->buddies, Buddy);
}

NateBuddyData* nateon_buddy_find_buddy_with_nid(NateBuddy* NBuddy, char* nid)
{
	GList* list;

	g_return_val_if_fail(nid != NULL, NULL);

	gaim_debug_info("nateon", "nbfb nid %s\n", nid);

	for(list = NBuddy->buddies; list != NULL; list = list->next)
	{
		NateBuddyData* buddy = (NateBuddyData*)list->data;

		if(!strcmp(buddy->nid, nid))
			return buddy;
	}

	return NULL;
}

NateBuddyData* nateon_buddy_find_buddy_with_uid(NateBuddy* NBuddy, char* uid)
{
	GList* list;

	g_return_val_if_fail(uid != NULL, NULL);

	gaim_debug_info("nateon", "nbfb uid %s\n", uid);
	
	for(list = NBuddy->buddies; list != NULL; list = list->next)
	{
		NateBuddyData* buddy = (NateBuddyData*)list->data;

		if(!strcmp(buddy->uid, uid))
			return buddy;
	}

	return NULL;
}

void nateon_buddy_append_group(NateBuddy* NBuddy, NateGroupData* Group)
{
	GaimGroup* gg = gaim_find_group(Group->name);
	
	if(!gg)
	{
		gg = gaim_group_new(Group->name);
		
		gaim_debug_info("nateon", "new group %s\n", Group->name);
		
		gaim_blist_add_group(gg, NULL);
	}
	
	if(!nateon_buddy_find_group_with_id(NBuddy, Group->gid))
	{
		gaim_debug_info("nateon", "append group %s\n", Group->name);
		
		NBuddy->groups = g_list_append(NBuddy->groups, Group);
	}

	gaim_blist_node_set_string((GaimBlistNode*)gg, "protocol", "nateon");
	gaim_blist_node_set_int((GaimBlistNode*)gg, "gid", Group->gid);
}

void nateon_buddy_remove_group(NateBuddy* NBuddy, NateGroupData* Group)
{
	NBuddy->groups = g_list_remove(NBuddy->groups, Group);
}

NateGroupData* nateon_buddy_find_group_with_id(NateBuddy* NBuddy, int gid)
{
	GList* list;

	g_return_val_if_fail(NBuddy != NULL, NULL);
	g_return_val_if_fail(gid    >= 0,    NULL);

	for(list = NBuddy->groups; list != NULL; list = list->next)
	{
		NateGroupData* group = list->data;

		if(group->gid == gid)
			return group;
	}

	return NULL;
}

NateGroupData* nateon_buddy_find_group_with_name(NateBuddy* NBuddy, char* name)
{
	GList* list;

	g_return_val_if_fail(NBuddy != NULL, NULL);
	g_return_val_if_fail(name   != NULL, NULL);

	for(list = NBuddy->groups; list != NULL; list = list->next)
	{
		NateGroupData* group = list->data;

		if((group->name) && !strcmp(name, group->name))
			return group;
	}

	return NULL;
}

void nateon_buddy_set_list_position(NateBuddyData* Buddy, NateonBuddyPosition position)
{
	Buddy->position = position;

	// gaim 리스트 처리
}

void nateon_buddy_set_status(NateBuddy* NBuddy, char* nid, NateonBuddyStatus status)
{
	NateBuddyData* Buddy = nateon_buddy_find_buddy_with_nid(NBuddy, nid);

	if(!Buddy) return;
	
	Buddy->status = status;

	gaim_prpl_got_user_status(NBuddy->account, Buddy->nid, nateon_buddy_get_status_id(status), NULL);
}

char* nateon_buddy_get_status_id(NateonBuddyStatus status)
{
	switch(status)
	{
		case NATEON_BUDDY_STAT_INVISIBLE:
			return "invisible";
		case NATEON_BUDDY_STAT_ONLINE:
			return "available";
		case NATEON_BUDDY_STAT_OFFLINE:
			return "offline";
		case NATEON_BUDDY_STAT_IDLE:
			return "idle";
		case NATEON_BUDDY_STAT_AWAY:
			return "away";
		case NATEON_BUDDY_STAT_BUSY:
			return "busy";
		case NATEON_BUDDY_STAT_MEETING:
			return "meeting";
		case NATEON_BUDDY_STAT_PHONE:
			return "phone";
		case NATEON_BUDDY_STAT_ERROR:
			return "error";
		case NATEON_BUDDY_STAT_MOBILE:
			return "mobile";
		case NATEON_BUDDY_STAT_MINIHP:
			return "minihp";
	};

	return NULL;
}

NateonBuddyStatus nateon_buddy_get_status(char* id)
{
	if(!id)
		return NATEON_BUDDY_STAT_ERROR;
	else if(!strcmp(id, "invisible"))
		return NATEON_BUDDY_STAT_INVISIBLE;
	else if(!strcmp(id, "available"))
		return NATEON_BUDDY_STAT_ONLINE;
	else if(!strcmp(id, "offline"))
		return NATEON_BUDDY_STAT_OFFLINE;
	else if(!strcmp(id, "away"))
		return NATEON_BUDDY_STAT_AWAY;
	else if(!strcmp(id, "idle"))
		return NATEON_BUDDY_STAT_IDLE;
	else if(!strcmp(id, "busy"))
		return NATEON_BUDDY_STAT_BUSY;
	else if(!strcmp(id, "meeting"))
		return NATEON_BUDDY_STAT_MEETING;
	else if(!strcmp(id, "phone"))
		return NATEON_BUDDY_STAT_PHONE;
	
	return NATEON_BUDDY_STAT_ERROR;
}

NateonBuddyPosition nateon_buddy_str_to_pos(char* strpos)
{
	NateonBuddyPosition position = 0x0000;

	if(strpos[0] == '1')
		position |= NATEON_BUDDY_POS_FOWARD;

	if(strpos[1] == '1')
		position |= NATEON_BUDDY_POS_ALLOW;

	if(strpos[1] == '1')
		position |= NATEON_BUDDY_POS_BLOCK;

	if(strpos[1] == '1')
		position |= NATEON_BUDDY_POS_REVERSE;

	return position;
}

NateonBuddyStatus nateon_buddy_str_to_stat(char* strstat)
{
	NateonBuddyStatus status = NATEON_BUDDY_STAT_OFFLINE;

	if(!strcmp(strstat, "O"))
		status = NATEON_BUDDY_STAT_ONLINE;
	else if(!strcmp(strstat, "A"))
		status = NATEON_BUDDY_STAT_AWAY;
	else if(!strcmp(strstat, "B"))
		status = NATEON_BUDDY_STAT_BUSY;
	else if(!strcmp(strstat, "P"))
		status = NATEON_BUDDY_STAT_PHONE;
	else if(!strcmp(strstat, "M"))
		status = NATEON_BUDDY_STAT_MEETING;
	else if(!strcmp(strstat, "X"))
		status = NATEON_BUDDY_STAT_INVISIBLE;

	return status;
}

void nateon_buddy_blist_load(NateBuddy* NBuddy)
{
	GaimBlistNode* gnode;
	GaimBlistNode* cnode;
	GaimBlistNode* bnode;

	gaim_debug_info("nateon", "load blist.xml\n");
	
	for(gnode = gaim_get_blist()->root; gnode; gnode = gnode->next)
	{
		GaimGroup*     gg = (GaimGroup*)gnode;
		int            gid;
		char*          gname;
		NateGroupData* group;

		const char* chk_protocol;

		if(!GAIM_BLIST_NODE_IS_GROUP(gnode))
			continue;

		chk_protocol = gaim_blist_node_get_string(gnode, "protocol");
		if(!chk_protocol || strcmp(chk_protocol, "nateon"))
			continue;

		group = g_new0(NateGroupData, 1);
		gid   = gaim_blist_node_get_int(gnode, "gid");
		gname = gg->name;

		group->gid  = gid;
		group->name = g_strdup(gg->name);

		gaim_debug_info("nateon", "group - %s - %d\n", gname, gid);
		
		NBuddy->groups = g_list_append(NBuddy->groups, group);
		
		for(cnode = gnode->child; cnode; cnode = cnode->next)
		{
			if(!GAIM_BLIST_NODE_IS_CONTACT(cnode))
				continue;

			for(bnode = cnode->child; bnode; bnode = bnode->next)
			{
				GaimBuddy*     gb = (GaimBuddy*)bnode;
				char*          uid;
				NateBuddyData* buddy;

				if(!GAIM_BLIST_NODE_IS_BUDDY(bnode))
					continue;

				chk_protocol = gaim_blist_node_get_string(gnode, "protocol");
				if(!chk_protocol || strcmp(chk_protocol, "nateon"))
					continue;
				
				buddy   = g_new0(NateBuddyData, 1);
				uid     = (char*)gaim_blist_node_get_string(bnode, "uid");

				buddy->uid   = g_strdup(uid);
				buddy->gid   = gid;
				buddy->isadd = FALSE;
				
				NBuddy->buddies = g_list_append(NBuddy->buddies, buddy);

				gb->proto_data = buddy;

				gaim_debug_info("nateon", "buddy - %s\n", uid);
			}
		}
	}
}

void nateon_buddy_blist_clear(NateBuddy* NBuddy)
{
	GaimBlistNode* gnode;
	GaimBlistNode* cnode;
	GaimBlistNode* bnode;

	for(gnode = gaim_get_blist()->root; gnode; gnode = gnode->next)
	{
		if(!GAIM_BLIST_NODE_IS_GROUP(gnode))
			continue;
		
		for(cnode = gnode->child; cnode; cnode = cnode->next)
		{
			if(!GAIM_BLIST_NODE_IS_CONTACT(cnode))
				continue;

			for(bnode = cnode->child; bnode; bnode = bnode->next)
			{
				if(!GAIM_BLIST_NODE_IS_BUDDY(bnode))
					continue;
				
				gaim_blist_remove_buddy((GaimBuddy*)bnode);
			}
		}

		gaim_blist_remove_group((GaimGroup*)gnode);
	}
}

char* nateon_passwd_encode(char* string)
{
	/*
	int i;
	GaimCipher *cipher;
	GaimCipherContext *context;
	guchar digest[16];
	gchar  passwd[33];

	cipher = gaim_ciphers_find_cipher("md5");
	context = gaim_cipher_context_new(cipher, NULL);

	gaim_cipher_context_append(context, (const guchar *)p_string, strlen(p_string));
	gaim_cipher_context_digest(context, sizeof(digest), digest, NULL);

	for (i = 0; i < 16; i++)
		g_snprintf(passwd + (i*2), 3, "%02x", digest[i]);

	gaim_cipher_context_destroy(context);

	e_string = g_strdup(passwd);
	*/

	int i;
	
	char enc_len[21][3] =
		{"00", "e6", "e5", "e4", "e3", "e2", "e1", "e0", "ef", "ee", "ed",
		       "ec", "eb", "ea", "e9", "e8", "f7", "f6", "f5", "f4", "f3"};

	char enc_str[128][3] = 
		{"00", "00", "00", "00", "00", "00", "00", "00", "00", "00", "00",
			   "00", "00", "00", "00", "00", "00", "00", "00", "00", "00",
			   "00", "00", "00", "00", "00", "00", "00", "00", "00", "00",
			   "00", "00", "6c", "5c", "4c", "3c", "2c", "1c", "0c", "fc",
			   "ec", "dc", "cc", "bc", "ac", "9c", "8c", "7d", "6d", "5d",
			   "4d", "3d", "2d", "1d", "0d", "fd", "ed", "dd", "cd", "bd",
			   "ad", "9d", "8d", "7a", "6a", "5a", "4a", "3a", "2a", "1a",
			   "0a", "fa", "ea", "da", "ca", "ba", "aa", "9a", "8a", "7b",
			   "6b", "5b", "4b", "3b", "2b", "1b", "0b", "fb", "eb", "db",
			   "cb", "bb", "ab", "9b", "8b", "0f", "68", "58", "48", "38",
			   "28", "18", "08", "f8", "e8", "d8", "c8", "b8", "a8", "98",
			   "88", "79", "69", "59", "49", "39", "29", "19", "09", "f9",
			   "e9", "d9", "c9", "b9", "a9", "99", "00"};
	
	char* buffer = g_new0(char, 49);
	
	strncpy(buffer, enc_len[strlen(string)], 2);
	
	for(i = 0; i < strlen(string); i++)
	{
		strncat(buffer, enc_str[(int)string[i]], 2);
	}

	for(i = strlen(string); i < 23; i++)
	{
		strncat(buffer, enc_str[0], 2);
	}

	return buffer;
}

char* nateon_passwd_decode(char* e_string)
{
	return "";
}
