#ifndef __BZ_BUDDY_H__
#define __BZ_BUDDY_H__

typedef enum _NateonBuddyStatus
{
	NATEON_BUDDY_STAT_INVISIBLE,
	NATEON_BUDDY_STAT_ONLINE,
	NATEON_BUDDY_STAT_OFFLINE,
	NATEON_BUDDY_STAT_AWAY,
	NATEON_BUDDY_STAT_IDLE,
	NATEON_BUDDY_STAT_BUSY,
	NATEON_BUDDY_STAT_MEETING,
	NATEON_BUDDY_STAT_PHONE,
	NATEON_BUDDY_STAT_ERROR,
	NATEON_BUDDY_STAT_MOBILE,
	NATEON_BUDDY_STAT_MINIHP
} NateonBuddyStatus;


typedef int NateonBuddyPosition;

#define NATEON_BUDDY_POS_NONE     0x0000 // None
#define NATEON_BUDDY_POS_FOWARD   0x1000 // Foward List
#define NATEON_BUDDY_POS_ALLOW    0x0100 // Allow List
#define NATEON_BUDDY_POS_BLOCK    0x0010 // Block List
#define NATEON_BUDDY_POS_REVERSE  0x0001 // Reverse List

typedef struct _NateGroupData
{
	int gid;
	char* name;
	gboolean visible;
} NateGroupData;

typedef struct _NateBuddyData
{
	char* nid;
	char* nick;
	char* name;
	char* email;
	char* phone;
	char* birth;
	char* uid;
	long  cid;
	NateonBuddyStatus status;
	NateonBuddyPosition position;
	int gid;

	gboolean isadd;
} NateBuddyData;


typedef struct _NateBuddy
{
	GaimAccount* account;
	int group_serial;
	NateBuddyData* me;
	GList* buddies;
	GList* groups;
} NateBuddy;

void nateon_buddy_destroy(NateBuddyData* Buddy);
void nateon_group_destroy(NateGroupData* Group);
void nateon_buddies_destroy(NateBuddy* NBuddy);

void nateon_buddy_append_buddy(NateBuddy* NBuddy, NateBuddyData* Buddy);
void nateon_buddy_remove_buddy(NateBuddy* NBuddy, NateBuddyData* Buddy);
void nateon_buddy_append_group(NateBuddy* NBuddy, NateGroupData* Group);
void nateon_buddy_remove_group(NateBuddy* NBuddy, NateGroupData* Group);

NateBuddyData* nateon_buddy_find_buddy_with_nid(NateBuddy* NBuddy, char* nid);
NateBuddyData* nateon_buddy_find_buddy_with_uid(NateBuddy* NBuddy, char* uid);
NateGroupData* nateon_buddy_find_group_with_id(NateBuddy* NBuddy, int gid);
NateGroupData* nateon_buddy_find_group_with_name(NateBuddy* NBuddy, char* name);

void nateon_buddy_set_list_position(NateBuddyData* Buddy, NateonBuddyPosition position);
void nateon_buddy_set_status(NateBuddy* NBuddy, char* nid, NateonBuddyStatus status);
char* nateon_buddy_get_status_id(NateonBuddyStatus status);
NateonBuddyStatus nateon_buddy_get_status(char* id);

NateonBuddyPosition nateon_buddy_str_to_pos(char* strpos);
NateonBuddyStatus nateon_buddy_str_to_stat(char* strstat);

void nateon_buddy_blist_load(NateBuddy* NBuddy);
void nateon_buddy_blist_clear(NateBuddy* NBuddy);

char* nateon_passwd_encode(char* string);
char* nateon_passwd_decode(char* string);

#endif
