#include "msgproc.h"
#include "buddy.h"

static struct NateMsg
{
	char* cmnd;
	void (*func)(NateConn* Nateon, char* cmnd, char* msgs);
}
NateAuthMsgs[] =
{
	{"PVER", nateon_cmnd_pver}, // 
	{"AUTH", nateon_cmnd_auth}, // 
	{"REQS", nateon_cmnd_reqs}, // ӿû
	{NULL, NULL}
},
NateMsgs[] =
{
	{"PING", nateon_cmnd_ping}, // 
	{"PONG", nateon_cmnd_ping}, // 
	{"LSIN", nateon_cmnd_lsin}, // äü
	{"CONF", NULL}, // "Nateon Today" html code
	{"TICK", NULL}, // XXX ??
	{"GLST", nateon_cmnd_glst}, // ׷
	{"LIST", nateon_cmnd_list}, // ģ
	{"CNIK", NULL}, // ȭ
	{"RENG", NULL}, // ׷
	{"ADDB", NULL}, // ģ߰
	{"RMVB", NULL}, // ģ
	{"RMVG", NULL}, // ׷
	{"INFY", nateon_cmnd_infy}, // ¶ģ
	{"NTFY", nateon_cmnd_infy}, // ģ
	{"NNIK", NULL}, // ϵģȭ
	{"ONST", nateon_cmnd_onst}, // 
	{"KILL", NULL}, // ٸ
	{"CTOC", NULL}, // Client To Client
	{"INVT", NULL}, // ȭʴ
	{"RESS", NULL}, // ȭǿû
	{"JOIN", NULL}, // ȭ Ӿ˸ ()
	{"ENTR", NULL}, // ȭ Ӿ˸ (ڽ)
	{"USER", NULL}, // ȭ ڸ
	{"MESG", NULL}, // äĿǵ
	{"WHSP", NULL}, // ??
	{"QUIT", NULL}, // ȭ  (,ڽ)
	{NULL, NULL}
};

void nateon_msg_table_build(NateConn* Nateon)
{
	int i;
	struct NateMsg* curtable = NULL;

	if(!Nateon || !Nateon->msg_table)
	{
		gaim_debug(GAIM_DEBUG_ERROR, "nateon", "Attempt to build a message table on a bogus structure\n");
		return;
	}

	switch(Nateon->curstep)
	{
		case NATEON_STEP_CONN:
			curtable = NateAuthMsgs;
			break;
		case NATEON_STEP_REQS:
			curtable = NateMsgs;
			break;
	};

	for(i = 0; curtable[i].cmnd; i++)
	{
		g_hash_table_insert(Nateon->msg_table,
						(gpointer)curtable[i].cmnd, (gpointer)&curtable[i]);
	}
}



void nateon_parse_msg(NateConn* Nateon, char* packet)
{
	struct NateMsg* msgent;
	gchar** token = g_strsplit(packet, " ", 3);

	if((msgent = g_hash_table_lookup(Nateon->msg_table, token[0])))
	{
		if(msgent->func)
		{
			(msgent->func)(Nateon, token[0], token[2]);
		}
		else
		{
			gaim_debug_info("nateon", "not supported %s\n", token[0]);
		}
	}
	else
	{
		gaim_debug_info("nateon", "unknown %s\n", token[0]);
	}

	g_strfreev(token);
}

void nateon_cmnd_pver(NateConn* Nateon, char* cmnd, char* msgs)
{
	nateon_set_current_step(Nateon, NATEON_STEP_AUTH);
	
	nateon_send_auth(Nateon);
}

void nateon_cmnd_auth(NateConn* Nateon, char* cmnd, char* msgs)
{
	nateon_set_current_step(Nateon, NATEON_STEP_REQS);

	nateon_send_reqs(Nateon);
}

void nateon_cmnd_reqs(NateConn* Nateon, char* cmnd, char* msgs)
{
	int error;
	GaimAccount* account = Nateon->account;
	GaimConnection* gc = gaim_account_get_connection(account);
	gchar** token = g_strsplit(msgs, " ", 0);

	g_hash_table_destroy(Nateon->msg_table);
	
	if(gc->inpa)
	{
		gaim_input_remove(gc->inpa);
	}
	
	if(Nateon->write_handler)
	{
		gaim_input_remove(Nateon->write_handler);
	}
	
	//g_free(Nateon->read_buffer);
	//gaim_circ_buffer_destroy(Nateon->write_buffer);
	
	close(Nateon->fd);

	Nateon->fd = -1;
	
	
	Nateon->host = token[1];
	Nateon->port = atoi(token[2]);

	//Nateon->write_buffer = gaim_circ_buffer_new(512);
	
	Nateon->msg_table = g_hash_table_new(g_str_hash, g_str_equal);
	nateon_msg_table_build(Nateon);

	error = gaim_proxy_connect(account, Nateon->host, Nateon->port,
										nateon_login_cb, gc);
	
	if (error || !gaim_account_get_connection(account))
	{
		gaim_connection_error(gc, _("Couldn't create socket"));
		return;
	}
}


void nateon_cmnd_ping(NateConn* Nateon, char* cmnd, char* msgs)
{
	nateon_send_ping(Nateon);
}

void nateon_cmnd_pong(NateConn* Nateon, char* cmnd, char* msgs)
{
	nateon_send_ping(Nateon);
}

void nateon_cmnd_lsin(NateConn* Nateon, char* cmnd, char* msgs)
{
	nateon_set_current_step(Nateon, NATEON_STEP_GLST);

	nateon_send_glst(Nateon);
}

void nateon_cmnd_glst(NateConn* Nateon, char* cmnd, char* msgs)
{
	// ׷
	gchar** token = g_strsplit(msgs, " ", 0);

	if(!token[1]) // ׷ ø 
	{
		char* group_serial = (char*)gaim_account_get_string(Nateon->account, "group_serial", "-1");

		if(!strcmp(group_serial, token[0])) // ׷츮Ʈ 
		{
			// blist.xml  ׷  о;
			nateon_buddy_blist_load(Nateon->buddy_list);
			
			nateon_set_current_step(Nateon, NATEON_STEP_LIST);
			
			nateon_send_list(Nateon);
		}
		else // ׷ ø ٲ  ۼ
		{
			gaim_account_set_string(Nateon->account, "group_serial", token[0]);

			nateon_buddy_blist_clear(Nateon->buddy_list);
		}
	}
	else // GLST 
	{
		if(token[2][0] == 'Y') // ׷
		{
			NateGroupData* group = g_new0(NateGroupData, 1);
			
			group->gid = atoi(token[3]);
			group->name = g_strdup(token[4]);
			
			nateon_buddy_append_group(Nateon->buddy_list, group);
		}
		else                   // ڱ׷
		{
			int groupid = atoi(token[4]);
			
			if(nateon_buddy_find_group_with_id(Nateon->buddy_list, groupid))
			{
				NateBuddyData* buddy = g_new0(NateBuddyData, 1);
				
				gaim_debug_info("nateon", "group %d\n", groupid);
				
				buddy->uid = g_strdup(token[3]);
				buddy->gid = groupid;
				buddy->isadd = FALSE;
				
				Nateon->buddy_list->buddies = g_list_append(Nateon->buddy_list->buddies, buddy);
			}
			else
			{
				gaim_debug_info("nateon", "unknown group %d\n", groupid);
			}

			if (atoi(token[0]) + 1 == atoi(token[1])) // GLST  Ѿ
			{
				nateon_set_current_step(Nateon, NATEON_STEP_LIST);

				nateon_send_list(Nateon);
			}
		}
	}

	g_free(token);
}

void nateon_cmnd_list(NateConn* Nateon, char* cmnd, char* msgs)
{
	// 
	gchar** token = g_strsplit(msgs, " ", 0);

	NateBuddyData* buddy = nateon_buddy_find_buddy_with_uid(Nateon->buddy_list, token[4]);
	
	if(buddy)
	{
		buddy->status = NATEON_BUDDY_STAT_OFFLINE;
		buddy->position = nateon_buddy_str_to_pos(token[2]);
		buddy->nid = g_strdup(token[3]);
		buddy->name = g_strdup(token[5]);
		buddy->nick = g_strdup(token[6]);
		buddy->phone = g_strdup(token[7]);
		buddy->email = g_strdup(token[8]);
		buddy->birth = g_strdup(token[9]);
		buddy->cid = atol(token[10]);
		
		nateon_buddy_append_buddy(Nateon->buddy_list, buddy);
	}
	else
	{
		gaim_debug_info("nateon", "unknown buddy %s\n", token[3]);
	}

	if(atoi(token[0]) + 1 == atoi(token[1]))
	{
		GaimAccount *account = Nateon->account;
		GaimConnection *gc = gaim_account_get_connection(account);
		
		nateon_set_current_step(Nateon, NATEON_STEP_CHSTAT);
		
		//ӽ -> nateon_send_onst  ̵ؾ
		gaim_connection_set_state(gc, GAIM_CONNECTED);
		//
	}

	g_free(token);
}

void nateon_cmnd_onst(NateConn* Nateon, char* cmnd, char* msgs)
{
	nateon_set_current_step(Nateon, NATEON_STEP_ONLINE);
}

void nateon_cmnd_infy(NateConn* Nateon, char* cmnd, char* msgs)
{
	gchar** token = g_strsplit(msgs, " ", 0);
	NateBuddyData* buddy = nateon_buddy_find_buddy_with_nid(Nateon->buddy_list, token[0]);
	NateonBuddyStatus status = nateon_buddy_str_to_stat(token[1]);
	
	nateon_buddy_set_status(Nateon->buddy_list, buddy->nid, status);
}

void nateon_send_ping(NateConn* Nateon)
{
	nateon_sendpacket(Nateon, "PING", NATEON_TRID_ZERO, NULL);
}

void nateon_send_auth(NateConn* Nateon)
{
	nateon_sendpacket(Nateon, "AUTH", NATEON_TRID_CUR, "DES");
}

void nateon_send_reqs(NateConn* Nateon)
{
	nateon_sendpacket(Nateon, "REQS", NATEON_TRID_CUR, "DES %s",
			gaim_account_get_username(Nateon->account));
}

void nateon_send_glst(NateConn* Nateon)
{
	char* group_serial;
	
	group_serial = (char*)gaim_account_get_string(Nateon->account, "group_serial", "-1");

	nateon_sendpacket(Nateon, "GLST", NATEON_TRID_CUR, group_serial);
}

void nateon_send_list(NateConn* Nateon)
{
	nateon_sendpacket(Nateon, "LIST", NATEON_TRID_CUR, NULL);
}
