#include "nateon.h"
#include "msgproc.h"

void nateon_login(GaimAccount* account)
{
	int error = 0;
	GaimConnection* gc     = gaim_account_get_connection(account);
	NateConn*       Nateon = g_new0(NateConn, 1);

	gc->proto_data  = Nateon;
	Nateon->account = account;

	Nateon->host = 
		g_strdup(gaim_account_get_string(account, "server", NATEON_SERVER));
	Nateon->port = gaim_account_get_int(account, "port", NATEON_PORT);

	Nateon->fd = -1;
	
	Nateon->write_buffer = gaim_circ_buffer_new(512);

	Nateon->buddy_list          = g_new0(NateBuddy, 1);
	Nateon->buddy_list->me      = g_new0(NateBuddyData, 1);
	Nateon->buddy_list->account = account;

	Nateon->msg_table = g_hash_table_new(g_str_hash, g_str_equal);
	nateon_msg_table_build(Nateon);

	nateon_set_current_step(Nateon, NATEON_STEP_CONN);
	
	error = gaim_proxy_connect(account, Nateon->host, Nateon->port,
										nateon_login_cb, gc);

	if (error || !gaim_account_get_connection(account))
	{
		gaim_connection_error(gc, _("Couldn't create socket"));
		return;
	}
}

void nateon_login_cb(gpointer data, gint source, GaimInputCondition cond)
{
	GaimConnection* gc     = data;
	NateConn*       Nateon = gc->proto_data;
	char* buf;

	if(source < 0)
	{
		gaim_connection_error(gc, _("Couldn't connect to host"));
		return;
	}

	if(!g_list_find(gaim_connections_get_all(), gc))
	{
		close(source);
		return;
	}

	Nateon->fd = source;
	
	switch(Nateon->curstep)
	{
		case NATEON_STEP_CONN:
			nateon_set_current_step(Nateon, NATEON_STEP_PVER);
			nateon_sendpacket(Nateon, "PVER", NATEON_TRID_SET, "3.540 3.0");
			break;
		case NATEON_STEP_REQS:
			buf = nateon_passwd_encode((char*)gaim_account_get_password(Nateon->account));
			
			nateon_set_current_step(Nateon, NATEON_STEP_LSIN);
			nateon_sendpacket(Nateon, "LSIN", NATEON_TRID_SET, "%s %s TE3 3.540 UTF8",
					gaim_account_get_username(Nateon->account), buf);
			g_free(buf);
			
			break;
	};

	gc->inpa = gaim_input_add(Nateon->fd, GAIM_INPUT_READ, nateon_read_cb, gc);
}

void nateon_read(NateConn* Nateon)
{
	char* cur = Nateon->read_buffer;
	char* end;

	while((cur < (Nateon->read_buffer + Nateon->read_bufused)) && !*cur)
	{
		cur++;
	}
	
	while(cur < Nateon->read_buffer + Nateon->read_bufused &&
			((end = strstr(cur, "\r\n")) || (end = strstr(cur, "\n"))))
	{
		int step = (*end == '\r'? 2:1);
		*end = (char)NULL;

		gaim_debug_info("nateon", "READ> %s\n", cur);
		nateon_parse_msg(Nateon, cur);
		
		cur = end + step;
	}

	if(cur != Nateon->read_buffer + Nateon->read_bufused)
	{
		Nateon->read_bufused -= (cur - Nateon->read_buffer);
		memmove(Nateon->read_buffer, cur, Nateon->read_bufused);
	}
	else
	{
		Nateon->read_bufused = 0;
	}
	
	return;
}

void nateon_read_cb(gpointer data, gint source, GaimInputCondition cond)
{
	GaimConnection* gc     = data;
	NateConn*       Nateon = gc->proto_data;
	int ret;

	if(Nateon->read_buflen < Nateon->read_bufused + NATEON_INIT_BUFSIZE)
	{
		Nateon->read_buflen += NATEON_INIT_BUFSIZE;
		Nateon->read_buffer =
			g_realloc(Nateon->read_buffer, Nateon->read_buflen);
	}
	
	ret = read(Nateon->fd, Nateon->read_buffer + Nateon->read_bufused,
							NATEON_INIT_BUFSIZE - 1);

	if(ret < 0 && errno == EAGAIN)
	{
		return;
	}
	else if(ret < 0)
	{
		gaim_connection_error(gc, _("Read error"));
		return;
	}
	else if(ret == 0)
	{
		gaim_connection_error(gc, _("Server has disconnected"));
		return;
	}

	Nateon->read_bufused += ret;
	Nateon->read_buffer[Nateon->read_bufused] = (char)NULL;

	nateon_read(Nateon);
}
void nateon_sendpacket(NateConn* Nateon,
						const char* cmnd, int trid, const char* format, ...)
{
	char* packet;
	char* _trid;
	char* msgs = NULL;
	va_list args;
	
	if(!cmnd || !strlen(cmnd)) return;
	
	_trid = nateon_get_trid(Nateon, trid);

	if(format)
	{
		va_start(args, format);
		msgs = g_strdup_vprintf(format, args);
		va_end(args);
	}
	
	if(msgs) packet = g_strdup_printf("%s %s %s\r\n", cmnd, _trid, msgs);
	else     packet = g_strdup_printf("%s %s\r\n", cmnd, _trid);

	nateon_write(Nateon, packet);

	g_free(packet);
	g_free(msgs);
	g_free(_trid);
}

void nateon_write(NateConn* Nateon, const char* buf)
{
	if(!Nateon->write_handler)
	{
		Nateon->write_handler = gaim_input_add(Nateon->fd, GAIM_INPUT_WRITE,
				nateon_write_cb, Nateon);
	}
	
	gaim_circ_buffer_append(Nateon->write_buffer, buf, strlen(buf));
}

void nateon_write_cb(gpointer data, gint source, GaimInputCondition cond)
{
	NateConn* Nateon = data;
	GaimConnection* gc = gaim_account_get_connection(Nateon->account);
	int ret;
	int writelen = gaim_circ_buffer_get_max_read(Nateon->write_buffer);
	char* packet = g_strndup(Nateon->write_buffer->outptr, writelen);
	packet[writelen] = (char)NULL;

	if(!writelen)
	{
		gaim_input_remove(Nateon->write_handler);
		Nateon->write_handler = 0;
		
		return;
	}

	ret = write(Nateon->fd, Nateon->write_buffer->outptr, writelen);

	gaim_debug_info("nateon", "SEND> %s", packet);
	g_free(packet);
	
	if(ret < 0 && errno == EAGAIN)
	{
		return;
	}
	else if(ret <= 0)
	{
		gaim_connection_error(gc, _("Server has disconnected"));
		return;
	}

	gaim_circ_buffer_mark_read(Nateon->write_buffer, ret);
}

void nateon_close(GaimConnection* gc)
{
	NateConn* Nateon = gc->proto_data;

	if(!Nateon)
	{
		return;
	}

	nateon_buddies_destroy(Nateon->buddy_list);

	g_hash_table_destroy(Nateon->msg_table);
	
	if(gc->inpa)
	{
		gaim_input_remove(gc->inpa);
	}
	
	if(Nateon->write_handler)
	{
		gaim_input_remove(Nateon->write_handler);
	}

	g_free(Nateon->read_buffer);
	gaim_circ_buffer_destroy(Nateon->write_buffer);

	g_free(Nateon);
}

const char* nateon_list_icon(GaimAccount* a, GaimBuddy* b)
{
	return "nateon";
}

void nateon_list_emblems(GaimBuddy* b, const char** se, const char** sw, const char** nw, const char** ne)
{
	NateBuddyData* buddy = b->proto_data;
	GaimPresence *presence = gaim_buddy_get_presence(b);
	const char* emblems[4] = {NULL, NULL, NULL, NULL};

	/*
	gaim_debug_info("nateon", "BZ - list_emblems\n");
	gaim_debug_info("nateon", "   - %s\n", buddy->uid);
	gaim_debug_info("nateon", "   - %s\n", nateon_buddy_get_status_id(buddy->status));
	*/
	
	if(!buddy || !gaim_presence_is_online(presence))
	{
		emblems[0] = "offline";
	}
	else
	{
		GaimStatusType *status_type = gaim_status_get_type(gaim_presence_get_active_status(gaim_buddy_get_presence(b)));
		GaimStatusPrimitive primitive = gaim_status_type_get_primitive(status_type);

		if(primitive > GAIM_STATUS_AVAILABLE)
		{
			emblems[0] = gaim_status_type_get_id(status_type);
		}

		/*
		gaim_debug_info("nateon", "   - %s\n", gaim_status_type_get_id(status_type));
		gaim_debug_info("nateon", "   - %s\n", gaim_status_type_get_name(status_type));
		*/
	}

	/*
	if(!buddy)
	{
		emblems[0] = "offline";
	}
	else
	{
		// 모바일접속
		// emblems[i++] = "wireless";
		// 차단아이콘
		// emblems[i++] = "nr";
	}
	*/

	*se = emblems[0];
	*sw = emblems[1];
	*nw = emblems[2];
	*ne = emblems[3];
}

void nateon_set_status(GaimAccount* account, GaimStatus* status)
{
	GaimConnection *gc = gaim_account_get_connection(account);
	NateConn* Nateon = NULL;
	//const char *status_id = gaim_status_get_id(status);

	gaim_debug_info("nateon", "BZ - in nateon_set_status\n");
	
	if(gc)
	{
		Nateon = gc->proto_data;
	}
}

GList* nateon_status_types(GaimAccount* account)
{
	GaimStatusType* type;
	GList* types = NULL;
	GaimValue *priority_value;

	priority_value = gaim_value_new(GAIM_TYPE_INT);
	gaim_value_set_int(priority_value, 1);

	type = gaim_status_type_new_with_attrs(GAIM_STATUS_AVAILABLE,
			nateon_buddy_get_status_id(NATEON_BUDDY_STAT_ONLINE),
			NULL, TRUE, TRUE, FALSE,
			"priority", _("Priority"), priority_value,
			"online", _("online"), gaim_value_new(GAIM_TYPE_STRING),
			NULL);
	types = g_list_append(types, type);

	type = gaim_status_type_new_with_attrs(GAIM_STATUS_INVISIBLE,
			nateon_buddy_get_status_id(NATEON_BUDDY_STAT_INVISIBLE),
			NULL, TRUE, TRUE, FALSE,
			"priority", _("Priority"), priority_value,
			"invisible", _("invisible"), gaim_value_new(GAIM_TYPE_STRING),
			NULL);
	types = g_list_append(types, type);

	type = gaim_status_type_new_with_attrs(GAIM_STATUS_AWAY,
			nateon_buddy_get_status_id(NATEON_BUDDY_STAT_AWAY),
			NULL, TRUE, TRUE, FALSE,
			"priority", _("Priority"), priority_value,
			"away", _("away"), gaim_value_new(GAIM_TYPE_STRING),
			NULL);
	types = g_list_append(types, type);

	type = gaim_status_type_new_with_attrs(GAIM_STATUS_AWAY,
			nateon_buddy_get_status_id(NATEON_BUDDY_STAT_IDLE),
			_("Idle"), TRUE, FALSE, FALSE,
			"priority", _("Priority"), priority_value,
			"away", _("idle"), gaim_value_new(GAIM_TYPE_STRING),
			NULL);
	types = g_list_append(types, type);

	type = gaim_status_type_new_with_attrs(GAIM_STATUS_UNAVAILABLE,
			nateon_buddy_get_status_id(NATEON_BUDDY_STAT_BUSY),
			_("Busy"), TRUE, TRUE, FALSE,
			"priority", _("Priority"), priority_value,
			"dnd", _("busy"), gaim_value_new(GAIM_TYPE_STRING),
			NULL);
	types = g_list_append(types, type);
	
	type = gaim_status_type_new_with_attrs(GAIM_STATUS_UNAVAILABLE,
			nateon_buddy_get_status_id(NATEON_BUDDY_STAT_MEETING),
			_("Meeting"), TRUE, TRUE, FALSE,
			"priority", _("Priority"), priority_value,
			"nr", _("meeting"), gaim_value_new(GAIM_TYPE_STRING),
			NULL);
	types = g_list_append(types, type);

	type = gaim_status_type_new_with_attrs(GAIM_STATUS_UNAVAILABLE,
			nateon_buddy_get_status_id(NATEON_BUDDY_STAT_PHONE),
			_("Phone"), TRUE, TRUE, FALSE,
			"priority", _("Priority"), priority_value,
			"nr", _("on the phone"), gaim_value_new(GAIM_TYPE_STRING),
			NULL);
	types = g_list_append(types, type);

	type = gaim_status_type_new_with_attrs(GAIM_STATUS_OFFLINE,
			nateon_buddy_get_status_id(NATEON_BUDDY_STAT_OFFLINE),
			_("Offline"), FALSE, TRUE, FALSE,
			"priority", _("Priority"), priority_value,
			"offline", _("offline"), gaim_value_new(GAIM_TYPE_STRING),
			NULL);
	types = g_list_append(types, type);

	return types;
}

static GList* nateon_actions(GaimPlugin* plugin, gpointer context)
{
	/*
	GaimConnection *gc = (GaimConnection *)context;
	GaimAccount *account;
	const char *user;

	GList *m = NULL;
	GaimPluginAction *act;

	act = gaim_plugin_action_new(_("Set Friendly Name..."),
								 nateon_show_set_friendly_name);
	m = g_list_append(m, act);
	
	return m;
	*/
	return NULL;
}

gboolean nateon_set_current_step(NateConn* Nateon, int step)
{
	GaimConnection* gc = gaim_account_get_connection(Nateon->account);

	int i;
	char progress_msg[NATEON_STEP][128] =
	{
		"NATE - CONN",
		"NATE - PVER",
		"NATE - AUTH",
		"NATE - REQS",
		"NATE - LSIN",
		"NATE - GLST",
		"NATE - LIST",
		"NATE - CHSTAT",
		"NATE - ONLINE",
	};
	
	for(i = 0; i < NATEON_STEP && progress_msg[i]; i++)
	{
		if(step == i)
		{
			gaim_connection_update_progress(gc, _(progress_msg[i]),
					step, NATEON_STEP);
			Nateon->curstep = step;

			gaim_debug_info("nateon", "BZ - status %d - %s\n", step, progress_msg[i]);
			return TRUE;
		}
	}
	
	return FALSE;
}

char* nateon_get_trid(NateConn* Nateon, int trid)
{
	char* ret;

	switch(trid)
	{
		case NATEON_TRID_ZERO:
			ret = g_strdup_printf("%d", 0);
			break;
		case NATEON_TRID_SET:
			Nateon->trid = 0;
		case NATEON_TRID_CUR:
		default:
			ret = g_strdup_printf("%d", Nateon->trid++);
			break;
	};

	return ret;
}

static GaimPluginProtocolInfo prpl_info =
{
	OPT_PROTO_MAIL_CHECK | OPT_PROTO_USE_POINTSIZE,
	NULL,					/* user_splits */
	NULL,					/* protocol_options */
	{"jpeg,gif,png,bmp", 0,0,96,96, GAIM_ICON_SCALE_DISPLAY},	/* icon_spec */
	nateon_list_icon,		/* list_icon */
	nateon_list_emblems,	/* list_emblems */
	NULL,					/* status_text */
	NULL,					/* tooltip_text */
	nateon_status_types,	/* away_states */
	NULL,					/* blist_node_menu */
	NULL,					/* chat_info */
	NULL,					/* chat_info_defaults */
	nateon_login,			/* login */
	nateon_close,			/* close */
	NULL,					/* send_im */
	NULL,					/* set_info */
	NULL,					/* send_typing */
	NULL,					/* get_info */
	nateon_set_status,		/* set_status */
	NULL,					/* set_idle */
	NULL,					/* change_passwd */
	NULL,					/* add_buddy */
	NULL,					/* add_buddies */
	NULL,					/* remove_buddy */
	NULL,					/* remove_buddies */
	NULL,					/* add_permit */
	NULL,					/* add_deny */
	NULL,					/* rem_permit */
	NULL,					/* rem_deny */
	NULL,					/* set_permit_deny */
	NULL,					/* join_chat */
	NULL,					/* reject_chat */
	NULL,					/* get_chat_name */
	NULL,					/* chat_invite */
	NULL,					/* chat_leave */
	NULL,					/* chat_whisper */
	NULL,					/* chat_send */
	NULL,					/* keepalive */
	NULL,					/* register_user */
	NULL,					/* get_cb_info */
	NULL,					/* get_cb_away */
	NULL,					/* alias_buddy */
	NULL,					/* group_buddy */
	NULL,					/* rename_group */
	NULL,					/* buddy_free */
	NULL,					/* convo_closed */
	gaim_normalize_nocase,	/* normalize */
	NULL,					/* set_buddy_icon */
	NULL,					/* remove_group */
	NULL,					/* get_cb_real_name */
	NULL,					/* set_chat_topic */
	NULL,					/* find_blist_chat */
	NULL,					/* roomlist_get_list */
	NULL,					/* roomlist_cancel */
	NULL,					/* roomlist_expand_category */
	NULL,					/* can_receive_file */
	NULL,					/* send_file */
	NULL,					/* new_xfer */
	NULL,					/* offline_message */
	NULL,					/* whiteboard_prpl_ops */
};


static GaimPluginInfo info =
{
	GAIM_PLUGIN_MAGIC,
	GAIM_MAJOR_VERSION,
	GAIM_MINOR_VERSION,
	GAIM_PLUGIN_PROTOCOL,                             /**< type           */
	NULL,                                             /**< ui_requirement */
	0,                                                /**< flags          */
	NULL,                                             /**< dependencies   */
	GAIM_PRIORITY_DEFAULT,                            /**< priority       */

	"prpl-nateon",                                    /**< id             */
	"NateON",                                         /**< name           */
	VERSION,                                          /**< version        */
	N_("NateON Protocol Plugin"),                     /**  summary        */
	N_("NateON Protocol Plugin"),                     /**  description    */
	"bugzero <bugzero@clus.org>",                     /**< author         */
	GAIM_WEBSITE,                                     /**< homepage       */

	NULL,                                             /**< load           */
	NULL,                                             /**< unload         */
	NULL,                                             /**< destroy        */

	NULL,                                             /**< ui_info        */
	&prpl_info,                                       /**< extra_info     */
	NULL,                                             /**< prefs_info     */
	NULL
	//nateon_actions
};

static void _init_plugin(GaimPlugin* plugin)
{
	GaimAccountOption    *option;

	option = 
		gaim_account_option_string_new(_("Server"), "server", NATEON_SERVER);
	prpl_info.protocol_options =
		g_list_append(prpl_info.protocol_options, option);
	
	option = 
		gaim_account_option_int_new(_("Port"), "port", NATEON_PORT);
	prpl_info.protocol_options = 
		g_list_append(prpl_info.protocol_options, option);

	gaim_prefs_remove("/plugins/prpl/nateon");
}

GAIM_INIT_PLUGIN(nateon, _init_plugin, info);
